--[[ CombatTime ]]

if not FixR then return end

local DefaultO = {
  ["framePoint"] = "CENTER";
  ["frameRelativeTo"] = "UIParent";
  ["frameRelativePoint"] = "CENTER";
  ["frameOffsetX"] = 0;
  ["frameOffsetY"] = 0;
  ["showCombatLogging"] = false;
  ["combatLoggingReminder"] = true;
  ["askIn_LFR"] = true;
  ["askIn_LFG"] = true;
  ["askIn_Raid"] = true;
  ["askIn_Party"] = true;
  ["continueEncounterWhenDead"] = true;
  ["useDBMPullDetection"] = true;
}
local misc = {
  ["desc"] = "Display combat length";
  ["alpha"] = true;
  ["GUI"] = {
    [1] = {
      ["type"] = "Boolean";
      ["opt"] = "showCombatLogging";
      ["desc"] = "display combat logging state";
    };
    [2] = {
      ["type"] = "Boolean";
      ["opt"] = "combatLoggingReminder";
      ["desc"] = "remind me to enable logging when entering an instance";
    };
    [3] = {
      ["type"] = "Boolean";
      ["opt"] = "askIn_LFR";
      ["desc"] = "show message in: LFR raid";
    };
    [4] = {
      ["type"] = "Boolean";
      ["opt"] = "askIn_LFG";
      ["desc"] = "show message in: LFG party";
    };
    [5] = {
      ["type"] = "Boolean";
      ["opt"] = "askIn_Scenario";
      ["desc"] = "show message in: Scenario";
    };
    [6] = {
      ["type"] = "Boolean";
      ["opt"] = "askIn_Raid";
      ["desc"] = "show message in: normal raid";
    };
    [7] = {
      ["type"] = "Boolean";
      ["opt"] = "askIn_Party";
      ["desc"] = "show message in: normal party";
    };
    [8] = {
      ["type"] = "Boolean";
      ["opt"] = "continueEncounterWhenDead";
      ["desc"] = "continue to count when your character died but the encounter is still in progress";
    };
    [9] = {
      ["type"] = "Boolean";
      ["opt"] = "useDBMPullDetection";
      ["desc"] = "also use DBM's pull detection to start a combat session";
    };
    [10] = {
      ["type"] = "Button";
      ["opt"] = "toggle moving";
      ["func"] = function(self) self:SetOption("move"); end;
    };
    [11] = {
      ["type"] = "Button";
      ["opt"] = "reset position";
      ["func"] = function(self) self:SetOption("reset"); end;
    };
  };
}
local function getOptionIndexByName(name)
  if misc and misc["GUI"] then
    for i, v in ipairs(misc["GUI"]) do
      if v["opt"] == name then
        return i
      end
    end
  end
  return 0
end

local M = FixR:NewModule("CombatTime", DefaultO, misc)

local frame = nil
local frameEvents = {};
local DBMEventHandler = {};
local moving = false
local absorbAmount = 0
local setOptionBoolean, getInstanceType, instanceTypeString

local function initFrame()
  frame = CreateFrame("Frame", "FixR_CombatTimeFrame", UIParent)
  frame:SetPoint(M.O["framePoint"], M.O["frameRelativeTo"], M.O["frameRelativePoint"], M.O["frameOffsetX"], M.O["frameOffsetY"])
  frame:SetFrameStrata("HIGH")
  frame:SetSize(220, 40)
  frame:SetScript("OnEvent", function(self, event, ...)
    frameEvents[event](self, ...)
  end);
  frame:SetScript("OnDragStart", function(self) self:StartMoving(); end);
  frame:SetScript("OnDragStop", function(self)
    self:StopMovingOrSizing()
    local point, relativeTo, relativePoint, xOfs, yOfs = self:GetPoint()
    M.O["framePoint"] = point or "CENTER"
    M.O["frameRelativeTo"] = relativeTo or "UIParent"
    M.O["frameRelativePoint"] = relativePoint or "CENTER"
    M.O["frameOffsetX"] = FixR:Round(xOfs, 4)
    M.O["frameOffsetY"] = FixR:Round(yOfs, 4)
  end);
  frame:SetScript("OnUpdate", function(self, elapsed)
    self.OnUpdate(self, elapsed)
  end);
  frame.timeSinceLastUpdate = 0
  frame.needsUpdate = false
  frame.inCombat = false
  frame.encounterInProgress = false
  frame.secondsInCombat = 0
  
  --CHAT_MSG_ADDON
  frame.COMprefix = "FixRCombatTime"
  frame.COMchannel = "RAID"
  frame.COMPlayerName = UnitName("player")
  frame.COMlastStatusRequestSent = -1
  local COMok = RegisterAddonMessagePrefix(frame.COMprefix)
  --FixR:Echo("CombatTime: Registering prefix... ", COMok and "|cffaaffaasuccessful" or "|cffff8888failed", ".")
  
  frame.combatLogging = LoggingCombat() and true or false
  
  frame.text = frame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
  frame.text:SetPoint("RIGHT")
  frame.text:SetText("CombatTime")
  frame.text:SetTextColor(1, 1, 1, 1)
  
  frame.startEncounter = function()
    frame.encounterInProgress = true
    frame.secondsInCombat = 0
    frame.timeSinceLastUpdate = 0
  end
  
  frame.OnUpdate = function(self, elapsed)
    self.timeSinceLastUpdate = self.timeSinceLastUpdate + elapsed
    while (self.encounterInProgress and self.timeSinceLastUpdate > 1) do --in seconds
      self.secondsInCombat = self.secondsInCombat + 1
      self.needsUpdate = true
      if self.inCombat or M.O["continueEncounterWhenDead"] and IsEncounterInProgress() then
        self.encounterInProgress = true
      else
        self.encounterInProgress = false
      end
      
      self.timeSinceLastUpdate = self.timeSinceLastUpdate - 1
    end
    
    if self.needsUpdate == true then
      self.needsUpdate = false
      self.updateDisplay()
    end
    
    if (self.COMlastStatusRequestSent >= 0) and (self.COMlastStatusRequestSent < (time() - 10)) then --wait 10 sec for reply then ask anyway
      self.COMlastStatusRequestSent = -1
      self:checkLogging()
    end
    
  end
  frame.updateDisplay = function()
    local l = frame.secondsInCombat
    local d = math.floor(l / 86400)
    l = l - 86400 * d
    local h = math.floor(l / 3600)
    l = l - 3600 * h
    local m = math.floor(l / 60)
    l = l - 60 * m
    local s = l
    
    local t
    
    if d > 0 then
      if m < 10 then
        m = "0"..m
      end
      if s < 10 then
        s = "0"..s
      end
      t = string.format("%dd %d:%s:%s", d, h, m, s)
    elseif h > 0 then
      if m < 10 then
        m = "0"..m
      end
      if s < 10 then
        s = "0"..s
      end
      t = string.format("%d:%s:%s", h, m, s)
    elseif m > 0 then
      if s < 10 then
        s = "0"..s
      end
      t = string.format("%d:%s", m, s)
    else
      t = s
    end
    if M.O["showCombatLogging"] then
      if frame.combatLogging then
        t = t.." |cff00ff00ON"
      else
        t = t.." |cffff0000OFF"
      end
    end
    frame.text:SetText(t)
  end
  frame.combatLoggingEvent = function(msg)
    --isLogging = LoggingCombat([newState])
    frame.combatLogging = LoggingCombat() and true or false
    frame.updateDisplay()
  end
  hooksecurefunc(SlashCmdList, "COMBATLOG", frame.combatLoggingEvent)
  
  
  --Chat hyperlink stuff
  frame.MyOnHyperlinkShow = function(chatFrame, link, text, button)
    if link:sub(1, 15) ~= "FixRCombatTime:" then
      return
    end
    local info = ChatTypeInfo["SYSTEM"]
    if link:sub(16) == "LogYES" then
      LoggingCombat(true)
      DEFAULT_CHAT_FRAME:AddMessage(COMBATLOGENABLED, info.r, info.g, info.b, info.id)
    elseif link:sub(16,20) == "LogNO" then
      LoggingCombat(false)
      DEFAULT_CHAT_FRAME:AddMessage(COMBATLOGDISABLED, info.r, info.g, info.b, info.id)
      local instanceType = link:sub(21)
      if instanceType == "LFR" then
        setOptionBoolean("askIn_LFR", 0, true)
      elseif instanceType == "LFG" then
        setOptionBoolean("askIn_LFG", 0, true)
      elseif instanceType == "SCEN" then
        setOptionBoolean("askIn_Scenario", 0, true)
      elseif instanceType == "NRAID" then
        setOptionBoolean("askIn_Raid", 0, true)
      elseif instanceType == "NPARTY" then
        setOptionBoolean("askIn_Party", 0, true)
      end
    --[[
    elseif link:sub(16) == "LogNO" then
      LoggingCombat(false)
      DEFAULT_CHAT_FRAME:AddMessage(COMBATLOGDISABLED, info.r, info.g, info.b, info.id)
    --]]
    end
    frame.combatLoggingEvent()
  end
  hooksecurefunc("ChatFrame_OnHyperlinkShow", frame.MyOnHyperlinkShow)
  local old_SetHyperlink = ItemRefTooltip.SetHyperlink -- http://forums.wowace.com/showthread.php?t=20217
	function ItemRefTooltip:SetHyperlink(link, ...)
		if link:sub(1, 15) == "FixRCombatTime:" then
			return
		end
		return old_SetHyperlink(self, link, ...)
	end
  
  function frame:checkLogging()
    if not self.lastCheckLogging then self.lastCheckLogging = time()-10 end
    if self.lastCheckLogging > (time() - 10) then
      return
    else
      self.lastCheckLogging = time()
    end
    
    self.COMlastStatusRequestSent = -1
    
    if M.O["combatLoggingReminder"] then
      if not LoggingCombat() then
        local instanceType = getInstanceType()
        if instanceType then
          local typeString = instanceTypeString[instanceType] or "unknown"
          FixR:Echo("Not logging combat, start now? \124cffaaffaa\124HFixRCombatTime:LogYES\124h[yes]\124h", " or \124cffff8888\124HFixRCombatTime:LogNO"..instanceType.."\124h[never in "..typeString.."]\124h")
        end
      --else
      --  FixR:Echo("CombatTime: |cff00ff00Logging combat.")
      end
    end
  end
  
  function frame:COMrequestStatus()
    if (M.O["combatLoggingReminder"]) and (not LoggingCombat()) then
      local inInstance, _ = IsInInstance()
      if inInstance then
        local blnAsk = false
        local instanceType = getInstanceType()
        if (instanceType == "LFR") and (M.O["askIn_LFR"]) then --LFR
          blnAsk = true
        elseif (instanceType == "LFG") and (M.O["askIn_LFG"]) then --LFG
          blnAsk = true
        elseif (instanceType == "SCEN") and (M.O["askIn_Scenario"]) then --Scenario
          blnAsk = true
        elseif (instanceType == "NRAID") and (M.O["askIn_Raid"]) then --10/25 man
          blnAsk = true
        elseif (instanceType == "NPARTY") and (M.O["askIn_Party"]) then --5 man
          blnAsk = true
        end
        if blnAsk then
          self.COMlastStatusRequestSent = time()
          SendAddonMessage(self.COMprefix, "statusreq", self.COMchannel)
        end
      end
    end
  end
  
  instanceTypeString = {
    ["LFR"] = "LFR";
    ["LFG"] = "LFG";
    ["SCEN"] = "Scenario";
    ["NRAID"] = "normal Raid";
    ["NPARTY"] = "normal Party";
  }
end

--[[
function frameEvents:UNIT_ABSORB_AMOUNT_CHANGED(unit)
  if unit == "player" then
    absorbAmount = UnitGetTotalAbsorbs("player")
    self.updateDisplay()
  end
end
--]]
function frameEvents:PLAYER_REGEN_DISABLED(...)
  if self.encounterInProgress == false then
    self.startEncounter()
  end
  self.inCombat = true
  self.updateDisplay()
end
function frameEvents:PLAYER_REGEN_ENABLED(...)
  self.inCombat = false
  self.updateDisplay()
end
function frameEvents:ZONE_CHANGED_NEW_AREA(...)
  --self:checkLogging()
  --request status when entering an instance
  self:COMrequestStatus()
end
--function frameEvents:GROUP_ROSTER_UPDATE(...)
  --print("GROUP_ROSTER_UPDATE")
  --self:checkLogging()
--end
--[[
function frameEvents:PARTY_MEMBERS_CHANGED(...)
end
function frameEvents:RAID_ROSTER_UPDATE(...)
end
--]]
function frameEvents:CHAT_MSG_ADDON(prefix, msg, channel, from)
  if prefix == self.COMprefix then
    if (msg == "statusreq") and (from ~= self.COMPlayerName) then
      if LoggingCombat() then
        --disable answer for now, since it's pretty much redundant
        --SendAddonMessage(self.COMprefix, "LogON", "WHISPER", from)
      end
    elseif msg == "LogON" then
      self:checkLogging()
    end
  end
end

function DBMEventHandler:pull(event, mod, delay, synced)
  if M.O["useDBMPullDetection"] then
    --if frame.encounterInProgress == false then
      --let DBM:pull always reset the combat session
      frame.startEncounter()
    --end
    frame.updateDisplay()
  end
end



function M:Load()
  if not frame then
    initFrame()
  end
  for k, v in pairs(frameEvents) do
    frame:RegisterEvent(k);
  end
  if DBM and DBM.RegisterCallback then
    for k, v in pairs(DBMEventHandler) do
      DBM:RegisterCallback(k, v)
    end
  end
  frame:Show()
  
  --You know this one time, at login...
  frame:COMrequestStatus()
end

function M:Unload()
  if frame then
    for k, v in pairs(frameEvents) do
      frame:UnregisterEvent(k);
    end
    frame:Hide()
  end
end

setOptionBoolean = function(optName, arg2, forceUpdateGUI)
  local bln, updateGUI = nil, true
  if arg2 == 1 then
    bln = true
    updateGUI = false
  elseif arg2 == 0 then
    bln = false
    updateGUI = false
  else --toggle
    bln = not (M.O[optName] or false)
  end
  M.O[optName] = bln
  FixR:Echo("CombatTime: "..optName..": "..(bln and "|cffaaffaaOn" or "|cffff8888Off"))
  if updateGUI or forceUpdateGUI then
    FixR:SetModuleGUIOption(M, getOptionIndexByName(optName), bln)
  end
  return bln, updateGUI --return <new_state>, <gui_needed_update>
end

getInstanceType = function()
  local _, instanceType, _, _, _, _, _, _ = GetInstanceInfo()
  if IsInGroup(LE_PARTY_CATEGORY_INSTANCE) then --LFR/LFG
    if instanceType == "raid" then --LFR
      return "LFR"
    elseif instanceType == "party" then --LFG
      return "LFG"
    elseif instanceType == "scenario" then
      return "SCEN"
    end
  elseif IsInRaid() then --10/25 man
    return "NRAID"
  else --elseif IsInGroup() then --5 man
    return "NPARTY"
  end
end

function M:SetOption(arg1, arg2)
  if arg1 == "move" then
    if moving then
      moving = false
      frame:SetMovable(false) --click-through
      frame:EnableMouse(false)
      frame:RegisterForDrag("")
    else
      moving = true
      frame:SetMovable(true);
      frame:EnableMouse(true);
      frame:RegisterForDrag("LeftButton");
    end
    FixR:Echo("CombatTime: now "..(moving and "moving" or "fixed"))
  elseif arg1 == "reset" then
    M.O["framePoint"] = DefaultO["framePoint"]
    M.O["frameRelativeTo"] = DefaultO["frameRelativeTo"]
    M.O["frameRelativePoint"] = DefaultO["frameRelativePoint"]
    M.O["frameOffsetX"] = DefaultO["frameOffsetX"]
    M.O["frameOffsetY"] = DefaultO["frameOffsetY"]
    
    frame:ClearAllPoints()
    frame:SetPoint(M.O["framePoint"], M.O["frameRelativeTo"], M.O["frameRelativePoint"], M.O["frameOffsetX"], M.O["frameOffsetY"])
  elseif arg1 == "showCombatLogging" then
    setOptionBoolean(arg1, arg2)
    if frame then
      frame.updateDisplay()
    end
  elseif arg1 == "combatLoggingReminder" then
    setOptionBoolean(arg1, arg2)
  elseif arg1 == "askIn_LFR" then
    setOptionBoolean(arg1, arg2)
  elseif arg1 == "askIn_LFG" then
    setOptionBoolean(arg1, arg2)
  elseif arg1 == "askIn_Scenario" then
    setOptionBoolean(arg1, arg2)
  elseif arg1 == "askIn_Raid" then
    setOptionBoolean(arg1, arg2)
  elseif arg1 == "askIn_Party" then
    setOptionBoolean(arg1, arg2)
  elseif arg1 == "continueEncounterWhenDead" then
    setOptionBoolean(arg1, arg2)
  elseif arg1 == "useDBMPullDetection" then
    setOptionBoolean(arg1, arg2)
  end
end
